@@ -7,6 +7,6 @@ typedef guint32 DWORD;
 typedef guint16 WORD;
 typedef guint8  BYTE;
 
-typedef gint32 UIN_T;
+typedef gchar *UIN_T;
 
 #endif /* __DATATYPE_H__ */
