@@ -12,6 +12,9 @@
 #include "showlist.h"
 
 #include <gtk/gtk.h>
+/* this is for the urgency hint stuff */
+#include <gdk/gdkx.h>
+
 
 /*** Global functions ***/
 guint flash_messages( void )
@@ -19,6 +22,7 @@ guint flash_messages( void )
 	static gboolean flash = FALSE;
 	static char prev_status = 0;
 	int have_message = 0;
+	XWMHints *hints;
 
 	GSList *contact;
 
@@ -34,23 +38,40 @@ guint flash_messages( void )
 			int type = ((STORED_MESSAGE_PTR)kontakt->stored_messages->data)->type;
 			have_message = TRUE;
 			if( flash )
-				set_contact_icon (kontakt, get_pixbuf_for_message (type));
+				gnomeicu_tree_set_contact_icon (kontakt, get_pixbuf_for_message (type));
 			else
-				set_contact_icon (kontakt, icon_blank_pixbuf);
+				gnomeicu_tree_set_contact_icon (kontakt, icon_blank_pixbuf);
 		}
 	}
 
 	if( have_message )
 	{
-		ready_set( flash );
+		gnomeicu_set_status_button (flash);
 	}
 
 	if( have_message != prev_status )
 	{
+
 		prev_status = have_message;
 		gtk_window_set_title (GTK_WINDOW (MainData->window),
 		                      have_message ? _("GnomeICU: Msgs") : "GnomeICU");
-		ready_set( have_message ? flash : FALSE );
+		gnomeicu_set_status_button (have_message ? flash : FALSE);
+
+
+		if (GTK_WIDGET_REALIZED(MainData->window)) {
+		  hints = XGetWMHints(GDK_WINDOW_XDISPLAY(MainData->window->window), 
+				      GDK_WINDOW_XWINDOW(MainData->window->window));
+		  if (have_message)
+		    hints->flags |= XUrgencyHint;
+		  else
+		    hints->flags &= ~XUrgencyHint;
+		  
+		  XSetWMHints(GDK_WINDOW_XDISPLAY(MainData->window->window), 
+			      GDK_WINDOW_XWINDOW(MainData->window->window),
+			      hints);
+		  XFree(hints);
+		}
+		
 	}
 
 	return TRUE;
