@@ -95,9 +95,7 @@ gboolean connection_alive = TRUE;
 
 _MainData *MainData;
 
-GtkWidget *app;
-
-static char *geometry;
+static char *geometry; /* geometry string */
 
 int preset_status = STATUS_ONLINE;
 
@@ -113,6 +111,7 @@ GSList *Groups = NULL;
 
 
 static gint pinger(gpointer data);
+static gint tray_not_yet(gpointer data);
 static void child_died (int);
 static void app_init (void);
 static gboolean is_already_running(void);
@@ -206,7 +205,7 @@ gnomeicu_set_status_button (gboolean flash)
 
   gtk_widget_show_all (status_im);
 
-  tray_update (flash);
+  tray_update ();
 }
 
 void
@@ -350,8 +349,8 @@ app_save_state_cb (GnomeClient *client, gint phase, GnomeRestartStyle save_style
                    gint shutdown, GnomeInteractStyle interact_style, gint fast,
                    gpointer client_data)
 {
-  gchar *argv[] = { client_data, "-s", NULL, "-g", NULL, NULL };
-  gint x, y;
+  gchar *argv[] = { client_data, "-s", NULL, "-g", NULL, NULL, NULL };
+  gint x, y, window_w, window_h;
 
   /* Set commands to clone and restart this application.  Note that we
      use the same values for both -- the session management code will
@@ -385,24 +384,30 @@ app_save_state_cb (GnomeClient *client, gint phase, GnomeRestartStyle save_style
     argv[2] = "offline";
   }
 
-  if (MainData->window != NULL) {
-    gint window_w, window_h;
-
-    gdk_drawable_get_size(GTK_WIDGET (MainData->window)->window,
-                          &window_w, &window_h);
-    preferences_set_int (PREFS_GNOMEICU_WINDOW_WIDTH, window_w);
-    preferences_set_int (PREFS_GNOMEICU_WINDOW_HEIGHT, window_h);
-
+  if (MainData->window == NULL) {
+    /* can't do anything to save state */
+    return TRUE;
   }
 
+  gdk_drawable_get_size(GTK_WIDGET (MainData->window)->window,
+			&window_w, &window_h);
+  preferences_set_int (PREFS_GNOMEICU_WINDOW_WIDTH, window_w);
+  preferences_set_int (PREFS_GNOMEICU_WINDOW_HEIGHT, window_h);
+
   if (MainData->hidden) {
     x = MainData->x;
     y = MainData->y;
+    argv[5] = "--hide";
   } else
     gtk_window_get_position (GTK_WINDOW (MainData->window), &x, &y);
 
+  preferences_set_int (PREFS_GNOMEICU_WINDOW_X, x);
+  preferences_set_int (PREFS_GNOMEICU_WINDOW_Y, y);
+
   argv[4] = g_strdup_printf ("+%d+%d", x ,y);
-  gnome_client_set_restart_command (client, 5, argv);
+
+  gnome_client_set_restart_command (client, 6, argv);
+
   g_free (argv[4]);
 
   return TRUE;
@@ -411,6 +416,7 @@ app_save_state_cb (GnomeClient *client, gint phase, GnomeRestartStyle save_style
 void app_init (void)
 {
 	GtkWidget *hbox;
+	GtkWidget *app_win;
 	BonoboDock* dock;
 	BonoboDockItem* dockitem;
         gint window_w, window_h;
@@ -419,17 +425,39 @@ void app_init (void)
 	if (MainData->xml == NULL)
 		exit(1);
 	
-	app = glade_xml_get_widget (MainData->xml, "app");
+	app_win = glade_xml_get_widget (MainData->xml, "app");
+
+	MainData->window = app_win;
 
         /* set main window size */
+
+	/* Revert to default if value is clearly wrong */
+	/* Otherwise it crashes and even brings down xnest with it */
+ beforewh:
         window_w = preferences_get_int (PREFS_GNOMEICU_WINDOW_WIDTH);
         window_h = preferences_get_int (PREFS_GNOMEICU_WINDOW_HEIGHT);
+	if (window_w > 5 * gdk_screen_get_width(gdk_screen_get_default()) ||
+	    window_h > 5 * gdk_screen_get_height(gdk_screen_get_default())) {
+	  preferences_unset(PREFS_GNOMEICU_WINDOW_WIDTH);
+	  preferences_unset(PREFS_GNOMEICU_WINDOW_HEIGHT);
+	  goto beforewh;
+	}
 
-	gtk_window_set_default_size (GTK_WINDOW (app), window_w, window_h);
+	MainData->x = preferences_get_int (PREFS_GNOMEICU_WINDOW_X);
+	MainData->y = preferences_get_int (PREFS_GNOMEICU_WINDOW_Y);
+	if (MainData->x > gdk_screen_get_width(gdk_screen_get_default()) ||
+	    MainData->y > gdk_screen_get_height(gdk_screen_get_default())) {
+	  MainData->x = 0;
+	  MainData->y = 0;
+	}
+	
+	if (MainData->x && MainData->y)
+	  gtk_window_move (GTK_WINDOW (app_win), MainData->x, MainData->y);
+	gtk_window_set_default_size (GTK_WINDOW (app_win), window_w, window_h);
 
 	if (geometry) {
-		if (!gtk_window_parse_geometry (GTK_WINDOW (app), geometry))
-			g_printerr (_("Invalid geometry string \"%s\"\n"), geometry);
+	  if (!gtk_window_parse_geometry (GTK_WINDOW (app_win), geometry))
+	    g_printerr (_("Invalid geometry string \"%s\"\n"), geometry);
 	}
 
 	dock = BONOBO_DOCK(glade_xml_get_widget (MainData->xml, "bonobodock1"));
@@ -441,20 +469,19 @@ void app_init (void)
 				| BONOBO_DOCK_ITEM_BEH_EXCLUSIVE;
 	gtk_widget_show(GTK_WIDGET(dock));
 	gtk_widget_show(glade_xml_get_widget (MainData->xml, "main_vbox"));
-	gnome_app_set_contents(GNOME_APP(app), glade_xml_get_widget (MainData->xml, "main_vbox"));
+	gnome_app_set_contents(GNOME_APP (app_win), glade_xml_get_widget (MainData->xml,"main_vbox"));
 
         /* setup tree view attributes */
         gnomeicu_tree_create ();
 
-	MainData->window = app;
 	MainData->notebook = glade_xml_get_widget (MainData->xml, "notebook");
 
-	g_signal_connect (G_OBJECT (app), "delete_event",
+	g_signal_connect (G_OBJECT (app_win), "delete_event",
 			  G_CALLBACK (icq_quit), NULL);
 	icon_themes_init();
 	emoticon_themes_init();
 	tray_init();
-	
+
 	hbox = glade_xml_get_widget (MainData->xml, "status_hbox");
 
 	status_button = gtk_button_new ();
@@ -488,15 +515,88 @@ int main( int argc, char *argv[] )
 
 	GnomeClient *client;
 	static char *cmdline_status, *cmdline_ip;
+	static gint hidden;
 
-	static struct poptOption arguments[] =
-	{
-		{"ipaddr", 'i', POPT_ARG_STRING, &cmdline_ip, 0, N_("Set IP address to send to ICQ"), N_("IP")},
-		{"status", 's', POPT_ARG_STRING, &cmdline_status, 0, N_("Initial status"), N_("STATUS")},
-		{"geometry", 'g', POPT_ARG_STRING, &geometry, 0, N_("X geometry specification (see \"X\" man page)"), N_("GEOMETRY")},
-		{NULL, 0, 0, NULL, 0, NULL, NULL}
+#ifdef GNOME_PARAM_GOPTION_CONTEXT
+
+	GOptionContext *goptcontext;
+	
+	const GOptionEntry options[] = {
+	  { "ipaddr",
+	    'i',
+	    0,
+	    G_OPTION_ARG_STRING,
+	    &cmdline_ip,
+	    N_("Set IP address to send to ICQ"),
+	    N_("<IP>")
+	  },
+	  { "status",
+	    's',
+	    0,
+	    G_OPTION_ARG_STRING,
+	    &cmdline_status,
+	    N_("Initial status"),
+	    N_("<STATUS>")
+	  },
+	  { "geometry",
+	    'g',
+	    0,
+	    G_OPTION_ARG_STRING,
+	    &geometry,
+	    N_("X geometry specification (see \"X\" man page)"),
+	    N_("<WIDTH>x<HEIGHT>+<X>+<Y>")
+	  },
+	  { "hide",
+	    'h',
+	    0,
+	    G_OPTION_ARG_NONE,
+	    &hidden,
+	    N_("Hide the GnomeICU main window upon startup"),
+	    NULL
+	  },
+	  { NULL, 0, 0, 0, NULL, NULL, NULL }
 	};
 
+#else /* GNOME_PARAM_GOPTION_CONTEXT */
+
+	static struct poptOption arguments[] = {
+		{ "ipaddr",
+		  'i',
+		  POPT_ARG_STRING,
+		  &cmdline_ip,
+		  0,
+		  N_("Set IP address to send to ICQ"),
+		  N_("IP")
+		},
+		{ "status",
+		  's',
+		  POPT_ARG_STRING,
+		  &cmdline_status,
+		  0,
+		  N_("Initial status"),
+		  N_("STATUS")
+		},
+		{ "geometry",
+		  'g',
+		  POPT_ARG_STRING,
+		  &geometry,
+		  0,
+		  N_("X geometry specification (see \"X\" man page)"),
+		  N_("WIDTHxHEIGHT+X+Y")
+		},
+		{ "hide",
+		  'h',
+		  POPT_ARG_NONE,
+		  &hidden,
+		  0,
+		  N_("Hide the GnomeICU main window upon startup"),
+		  NULL
+		},
+		{ NULL, 0, 0, NULL, 0, NULL, NULL }
+        };
+
+#endif /* GNOME_PARAM_GOPTION_CONTEXT */
+
 	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
@@ -506,8 +606,6 @@ int main( int argc, char *argv[] )
 #endif
 
 
-
-
 	srand( time( NULL ) );
 
 	our_info = g_new0( USER_INFO_STRUCT, 1 );
@@ -520,10 +618,20 @@ int main( int argc, char *argv[] )
 
 	configfilename = g_strdup( "/GnomeICU/" );
 
+#ifdef GNOME_PARAM_GOPTION_CONTEXT
+	goptcontext = g_option_context_new (_("GnomeICU ICQ Instant Messenger"));
+	
+	g_option_context_add_main_entries (goptcontext, options, GETTEXT_PACKAGE);
+#endif
+	
 	gnome_program_init (PACKAGE, VERSION,
 			    LIBGNOMEUI_MODULE,
 			    argc, argv,
+#ifdef GNOME_PARAM_GOPTION_CONTEXT
+			    GNOME_PARAM_GOPTION_CONTEXT, goptcontext,
+#else
 			    GNOME_PARAM_POPT_TABLE, arguments,
+#endif
 			    GNOME_PARAM_GNOME_DATADIR, GNOMEICU_DATADIR,
 			    GNOME_PARAM_HUMAN_READABLE_NAME, _("GnomeICU"),
 			    GNOME_PARAM_APP_DATADIR, GNOMEICU_DATADIR,
@@ -581,6 +689,7 @@ int main( int argc, char *argv[] )
 	signal(SIGCHLD, child_died);
 
 	client = gnome_master_client();
+
 	g_signal_connect (G_OBJECT (client), "die",
 	                  G_CALLBACK (icq_quit),
 	                  NULL );
@@ -590,7 +699,7 @@ int main( int argc, char *argv[] )
 	                  argv[0]);
 
 	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-					  "gnome-gnomeicu.png", TRUE, NULL);
+					  "gnomeicu.png", TRUE, NULL);
 	if (file != NULL) {
 	  gtk_window_set_default_icon_from_file( file, NULL );
 	  g_free (file);
@@ -600,12 +709,21 @@ int main( int argc, char *argv[] )
         /* populate loaded data to the tree model */
 	gnomeicu_tree_populate_data ();
 
-	gtk_widget_show(app);
+	/* see if we want to start up hidden */
+	if ( hidden ) {
+	  MainData->hidden = TRUE;
+	  if (!tray_exists()) 
+	    g_timeout_add (15*1000, (GtkFunction) tray_not_yet, NULL );
+	  
+	} else {
+	  gtk_widget_show(MainData->window);
+	  MainData->hidden = FALSE;
+	}
 
  	while( gtk_events_pending() )
  		gtk_main_iteration();
 
-	/* disabled file transfer create_tcp_line(); */
+	create_tcp_line();
 
         if (is_new_user)
           v7_new_user(passwd);
@@ -662,13 +780,13 @@ void
 gnomeicu_about (GtkWidget *widget, gpointer data)
 {
   static GtkWidget *about = NULL;
-  GdkPixbuf *pixbuf = NULL;
+  static GdkPixbuf *pixbuf = NULL;
   gchar *file;
 
   /* Authors Credits */
   const gchar *authors[] = {
 
-    "Olivier Crete <tester@tester.ca>",
+    "Olivier Cr\xc3\xaate <tester@tester.ca>",
     "Gediminas Paulauskas <menesis@delfi.lt>",
     "Patrick Sung <iam@patricksung.com>",
     "David Tabachnikov <captain@bezeqint.net>",
@@ -679,39 +797,40 @@ gnomeicu_about (GtkWidget *widget, gpointer data)
 
   /* Documenters Credits */
   const gchar *documenters[] = {
+   "Thomas Canty <tommydal@ihug.com.au>",
+   "Ilya Konstantinov <gnomeicu-docs@future.galanet.net>",
+   "Christopher Ness <nesscg@mcmaster.ca>",
     NULL
   };
 
   /* Translator credits */
   gchar *translator_credits = _("translator_credits");
 
-  if (about) {
-    gtk_window_present (GTK_WINDOW(about));
-    return;
+
+  if (pixbuf == NULL) {
+    file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
+				      "gnomeicu.png", TRUE, NULL);
+    if (file != NULL) {
+      pixbuf = gdk_pixbuf_new_from_file (file, NULL);
+      g_free (file);
+    }
   }
 
-  file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-                                    "gnome-gnomeicu.png", TRUE, NULL);
-  if (file != NULL) {
-    pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-    g_free (file);
-  }
+  gtk_show_about_dialog(GTK_WINDOW(MainData->window),
+			"authors", authors,
+			"comments", _("GnomeICU is a small, fast and functional "
+				      "clone of Mirabilis' ICQ program, specifically "
+				      "designed for GNOME."),
+			"copyright",  "Copyright \xc2\xa9 1999-2006 Jeremy Wise, Olivier Cr\xc3\xaate, Patrick Sung",
+			"documenters", documenters,
+			"license", _("This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.") ,
+			"wrap-license", TRUE,
+			"logo", pixbuf, 
+			"version", VERSION,
+			"website", "http://gnomeicu.sourceforge.net/",
+			NULL);
+  //			"logo-icon-name", "gnomeicu.png",
 
-  about = gnome_about_new ("GnomeICU", VERSION,
-                           "Copyright \xc2\xa9 1999-2004 Jeremy Wise, Olivier Cr\xc3\xaate, Patrick Sung",
-                           _("GnomeICU is a small, fast and functional "
-                             "clone of Mirabilis' ICQ program, specifically "
-                             "designed for GNOME."),
-                           authors,
-                           documenters,
-                           ((strcmp (translator_credits, "translator_credits") != 0) ? translator_credits : NULL),
-                           pixbuf);
-
-  gtk_widget_show (about);
-
-  g_object_add_weak_pointer (G_OBJECT (about), (gpointer *)&about);
-
-  return;
 }
 
 void go_url_home (void)
@@ -727,7 +846,7 @@ void go_url_updates (void)
 void icq_quit( GtkWidget *widget, gpointer data )
 {
 	GSList *contact;
-        gint window_w, window_h;
+        gint window_w, window_h, x, y;
 
 #ifdef TRACE_FUNCTION
 	g_print( "icq_quit\n" );
@@ -749,6 +868,14 @@ void icq_quit( GtkWidget *widget, gpointer data )
                 preferences_set_int (PREFS_GNOMEICU_WINDOW_WIDTH, window_w);
                 preferences_set_int (PREFS_GNOMEICU_WINDOW_HEIGHT, window_h);
 
+		if (MainData->hidden) {
+		  x = MainData->x;
+		  y = MainData->y;
+		} else
+		  gtk_window_get_position (GTK_WINDOW (MainData->window),
+					   &x, &y);
+		preferences_set_int (PREFS_GNOMEICU_WINDOW_X, x);
+		preferences_set_int (PREFS_GNOMEICU_WINDOW_Y, y);
 	}
 	g_object_unref (G_OBJECT (MainData->xml));
 
@@ -850,3 +977,12 @@ void gtkspell_change_status(void)
 }
 
 #endif
+
+gint tray_not_yet(gpointer data)
+{
+  if (!GTK_WIDGET_VISIBLE(MainData->window) && !tray_exists()) {
+    gtk_widget_show(MainData->window);
+    MainData->hidden = FALSE;
+  }
+  return FALSE;
+}
