@@ -60,6 +60,7 @@ GdkPixbuf *icon_rename_pixbuf;
 GdkPixbuf *icon_group_pixbuf;
 GdkPixbuf *icon_lists_pixbuf;
 GdkPixbuf *icon_groups_pixbuf;
+GdkPixbuf *icon_birthday_pixbuf;
 
 static gboolean icon_themes_initialized = FALSE;
 /*** Local functions ***/
@@ -131,7 +132,7 @@ icon_themes_refresh (void)
 {
   icon_themes_load ();
   gnomeicu_tree_refresh_visuals ();
-  ready_set (FALSE);
+  gnomeicu_set_status_button (FALSE);
   Shortcut_Update_All ();
   gnomeicu_spinner_reload ();
 }
@@ -175,6 +176,7 @@ icon_themes_load (void)
   icon_group_pixbuf = init_one_pixmap ("gnomeicu-group.png");
   icon_lists_pixbuf = init_one_pixmap ("gnomeicu-lists.png");
   icon_groups_pixbuf = init_one_pixmap ("gnomeicu-groups.png");
+  icon_birthday_pixbuf = init_one_pixmap ("gnomeicu-birthday.png");
 }
 
 GdkPixbuf *get_pixbuf_for_status (DWORD status)
