@@ -22,6 +22,8 @@
 #include <libgnome/libgnome.h>
 #include <gtk/gtk.h>
 #include <string.h>
+/* this is for the urgency hint stuff */
+#include <gdk/gdkx.h>
 
 #ifdef WITH_GTKSPELL
 # include <gtkspell/gtkspell.h>
@@ -332,15 +334,32 @@ void open_message_dlg_with_message (Contact_Member *contact,
 
 	if (contact->msg_dlg_xml != NULL) {
           xml = contact->msg_dlg_xml;
-          if (message_text && contact->newmsg_blink_timeout == 0 &&
-              gtk_window_get_focus(GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog"))) &&
-              !GTK_WIDGET_HAS_FOCUS(gtk_window_get_focus(GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog")))))
-            contact->newmsg_blink_timeout = gtk_timeout_add(500, (GtkFunction) msg_blink_timeout, contact);
+	  msg_dlg = glade_xml_get_widget (xml, "message_dialog");
+	  
+	  if (message_text && contact->newmsg_blink_timeout == 0 &&
+	      gtk_window_get_focus(GTK_WINDOW(msg_dlg)) &&
+	      !GTK_WIDGET_HAS_FOCUS(gtk_window_get_focus(GTK_WINDOW(msg_dlg)))){
+	    XWMHints *hints;
+	    contact->newmsg_blink_timeout = gtk_timeout_add(500,
+							    (GtkFunction) msg_blink_timeout, 
+							    contact);
+	    
+	    if (GTK_WIDGET_REALIZED(msg_dlg)) {
+	      hints = XGetWMHints(GDK_WINDOW_XDISPLAY(msg_dlg->window), 
+				  GDK_WINDOW_XWINDOW(msg_dlg->window));
+	      hints->flags |= XUrgencyHint;
+	      XSetWMHints(GDK_WINDOW_XDISPLAY(msg_dlg->window), 
+			  GDK_WINDOW_XWINDOW(msg_dlg->window), hints);
+	      XFree(hints);
+	    }
+	  }
 	} else {
           xml = gicu_util_open_glade_xml ("message.glade", "message_dialog");
           if (xml == NULL)
             return;
 
+	  msg_dlg = glade_xml_get_widget (xml, "message_dialog");
+
           contact->msg_dlg_xml = xml;
 
           chat_toggle = glade_xml_get_widget (xml, "chat_toggle");
@@ -354,7 +373,8 @@ void open_message_dlg_with_message (Contact_Member *contact,
           g_signal_connect (conversation, "button_press_event",
                             G_CALLBACK(msg_url_text_button_pressed), NULL);
 
-          gtk_window_set_type_hint (GTK_WINDOW(glade_xml_get_widget (xml, "message_dialog")), GDK_WINDOW_TYPE_HINT_NORMAL);
+          gtk_window_set_type_hint (GTK_WINDOW(msg_dlg),
+				    GDK_WINDOW_TYPE_HINT_NORMAL);
 
           /* Set the tags table to the global one */
           if ( ! message_tags )
@@ -370,12 +390,11 @@ void open_message_dlg_with_message (Contact_Member *contact,
           gtk_text_buffer_create_mark(text_buffer, "insert_mark",
                                       &end_iter, FALSE);
 
-	  gtk_window_set_role(GTK_WINDOW(glade_xml_get_widget(xml, "message_dialog")), "GnomeICU_Message");
+	  gtk_window_set_role(GTK_WINDOW(msg_dlg), "GnomeICU_Message");
 
           glade_xml_signal_autoconnect (xml);
 	}
 
-	msg_dlg = glade_xml_get_widget (xml, "message_dialog");
         set_window_icon( msg_dlg, "gnomeicu-message.png");
 	gtk_window_set_title (GTK_WINDOW (msg_dlg), contact->nick);
 	chat_toggle = glade_xml_get_widget (xml, "chat_toggle");
@@ -464,10 +483,19 @@ gboolean msg_blink_timeout(Contact_Member *contact)
 {
   GtkWidget *msg_dlg = glade_xml_get_widget (contact->msg_dlg_xml, 
 					     "message_dialog");
+  XWMHints *hints;
 
   if (!contact->newmsg_blink_timeout) {
     contact->newmsg_blink = 0;
     set_window_icon( msg_dlg, "gnomeicu-message.png");
+    if (GTK_WIDGET_REALIZED(msg_dlg)) {
+      hints = XGetWMHints(GDK_WINDOW_XDISPLAY(msg_dlg->window), 
+			  GDK_WINDOW_XWINDOW(msg_dlg->window));
+      hints->flags &= ~XUrgencyHint;
+      XSetWMHints(GDK_WINDOW_XDISPLAY(msg_dlg->window), 
+		  GDK_WINDOW_XWINDOW(msg_dlg->window), hints);
+      XFree(hints);
+    }
     return FALSE;
   }
   
@@ -506,7 +534,7 @@ void send_message (GtkWidget *dlg)
 	if (contact == NULL)
 		return;
 
-	if (!is_connected(GTK_WINDOW(dlg), _("You can not send messages while being disconnected.")))
+	if (!is_connected(GTK_WINDOW(dlg), _("You can not send messages while disconnected.")))
 		return;
 
 	entry = glade_xml_get_widget (contact->msg_dlg_xml, "input");
@@ -620,15 +648,18 @@ message_input_key_callback (GtkWidget *widget, GdkEventKey *event)
 {
 
 #ifdef TRACE_FUNCTION
-	g_print("message_input_key_callback\n");
+  g_print("message_input_key_callback\n");
 #endif
 
-	if ((event->state & GDK_CONTROL_MASK) && (event->keyval == GDK_Return)) {
-		send_message (g_object_get_data (G_OBJECT (widget), "message_dialog"));
-		return TRUE;
-	}
+  if ((event->keyval == GDK_Return || event->keyval == GDK_KP_Enter) && 
+      (!(event->state & (GDK_CONTROL_MASK|GDK_SHIFT_MASK)) ^
+       !preferences_get_bool(PREFS_GNOMEICU_ENTER_SENDS))) {
 
-	return FALSE;
+    send_message (g_object_get_data (G_OBJECT (widget), "message_dialog"));
+    return TRUE;
+  }
+  
+  return FALSE;
 }
 
 void message_usermenu_clicked_callback (GtkWidget *widget, gpointer data)
@@ -842,6 +873,9 @@ void message_dialog_focus_callback(GtkWidget *widget, gpointer data)
 {
 
   Contact_Member *contact = g_object_get_data (G_OBJECT (widget), "contact");
+  GtkWidget *msg_dlg = glade_xml_get_widget (contact->msg_dlg_xml, 
+					     "message_dialog");
+  XWMHints *hints;
 
   if (contact->newmsg_blink_timeout) {
     gtk_timeout_remove(contact->newmsg_blink_timeout);
@@ -849,6 +883,15 @@ void message_dialog_focus_callback(GtkWidget *widget, gpointer data)
     contact->newmsg_blink_timeout = 0;
     contact->newmsg_blink = TRUE;
 
+    set_window_icon( msg_dlg, "gnomeicu-message.png");
+
+    hints = XGetWMHints(GDK_WINDOW_XDISPLAY(msg_dlg->window), 
+			GDK_WINDOW_XWINDOW(msg_dlg->window));
+    hints->flags &= ~XUrgencyHint;
+    XSetWMHints(GDK_WINDOW_XDISPLAY(msg_dlg->window), 
+		GDK_WINDOW_XWINDOW(msg_dlg->window), hints);
+    XFree(hints);
+
     msg_blink_timeout(contact);
   }
 }
@@ -863,3 +906,10 @@ gboolean on_message_dialog_key_press_event(GtkDialog *dlg, GdkEventKey *event, g
   return FALSE;
 }
 
+gboolean msg_focus_in_event(GtkWidget *input, GdkEventFocus *focus, 
+			    GtkWidget *conversation)
+{
+  gtk_widget_grab_focus (input);
+
+  return TRUE;
+}
