@@ -209,6 +209,65 @@ void update_personal_info ( UIN_T uin )
 	if( kontakt->info->last != NULL )
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "lname_entry")), kontakt->info->last);
 
+	/*	if( kontakt->idletime > 0) {
+	  char timebuf[100];
+	  strftime(timebuf, 100, "%X", gmtime((time_t*)&kontakt->idletime));
+	  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "idle_entry")), timebuf);
+	} else
+	  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "idle_entry")), _("Not Idle"));
+	*/
+
+	if( kontakt->status == STATUS_OFFLINE) 
+	  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "onlinesince_entry")), _("Offline"));
+	else {
+	  char timebuf[100];
+	  strftime(timebuf,100,"%c",localtime((time_t*)&kontakt->online_since));
+	  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "onlinesince_entry")), timebuf);
+	}
+	  
+	if( kontakt->member_since) {
+	  char timebuf[100];
+	  strftime(timebuf,100,"%c",localtime((time_t*)&kontakt->member_since));
+	  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "membersince_entry")), timebuf);
+	  gtk_widget_show(glade_xml_get_widget (kontakt->info_dlg_xml, "membersince_label"));
+	  gtk_widget_show(glade_xml_get_widget (kontakt->info_dlg_xml, "membersince_entry"));
+	} else if (GTK_WIDGET_VISIBLE(glade_xml_get_widget (kontakt->info_dlg_xml, "membersince_label"))) {
+	  gtk_widget_hide(glade_xml_get_widget (kontakt->info_dlg_xml, "membersince_label"));
+	  gtk_widget_hide(glade_xml_get_widget (kontakt->info_dlg_xml, "membersince_entry"));
+	}
+
+	if ( kontakt->idle_since) {
+	  int idlefor = (time(NULL) - kontakt->idle_since)/60;
+	  gchar *timestr;
+	  if (kontakt->idle_since < 60)
+	    timestr = g_strdup_printf(_("%d minutes"), idlefor); 
+	  else if (kontakt->idle_since < 24*60)
+	    timestr = g_strdup_printf(_("%d hours %d minutes"), 
+				      idlefor / 60, 
+				      idlefor % 60); 
+	  else
+	    timestr = g_strdup_printf(_("%d days %d hours %d minutes"), 
+				      idlefor / (60*24),
+				      (idlefor % (60*24))/60, 
+				      idlefor % 60); 
+	  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "idlefor_entry")), timestr);
+	  g_free(timestr);
+	  gtk_widget_show(glade_xml_get_widget (kontakt->info_dlg_xml, "idlefor_label"));
+	  gtk_widget_show(glade_xml_get_widget (kontakt->info_dlg_xml, "idlefor_entry"));
+	} else if (GTK_WIDGET_VISIBLE(glade_xml_get_widget (kontakt->info_dlg_xml, "idlefor_label"))){
+	  gtk_widget_hide(glade_xml_get_widget (kontakt->info_dlg_xml, "idlefor_label"));
+	  gtk_widget_hide(glade_xml_get_widget (kontakt->info_dlg_xml, "idlefor_entry"));
+	}
+	if (kontakt->status == STATUS_OFFLINE)
+	  if( kontakt->last_seen) {
+	    char timebuf[100];
+	    strftime(timebuf, 100, "%c", localtime(&kontakt->last_seen));
+	    gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "lastseen_entry")), timebuf);
+	  } else
+	    gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "lastseen_entry")), _("Never"));
+	else
+	  gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "lastseen_entry")), _("Now"));
+
 	if( kontakt->info->sex == MALE )
 		gtk_label_set_text (GTK_LABEL (glade_xml_get_widget (kontakt->info_dlg_xml, "sex_entry")), _("Male"));
 	else if( kontakt->info->sex == FEMALE )
@@ -441,7 +500,7 @@ gboolean userinfo_refresh_clicked_callback (GtkWidget *widget, gpointer data)
 
        uin = g_strdup (g_object_get_data (G_OBJECT (widget), "uin"));
 
-       if (!is_connected(GTK_WINDOW(g_object_get_data (G_OBJECT (widget), "dlg")), _("You can not update user informations while being disconnected.")))
+       if (!is_connected(GTK_WINDOW(g_object_get_data (G_OBJECT (widget), "dlg")), _("You can not update user information while disconnected.")))
              return TRUE;
         
        v7_request_info(mainconnection, uin);
