@@ -18,6 +18,7 @@
 #include "tray.h"
 #include "util.h"
 #include "v7send.h"
+#include "gnomecfg.h"
 
 #include <gtk/gtk.h>
 #include <string.h>
@@ -75,7 +76,7 @@ void User_Offline( GSList *contact )
 
 	g_assert(contact != NULL);
 
-	set_contact_status (contact, STATUS_OFFLINE);
+	set_contact_status (contact, STATUS_OFFLINE, FALSE);
 	if( kontakt->sok )
 	{
 		if( kontakt->gioc )
@@ -93,12 +94,17 @@ void User_Offline( GSList *contact )
 
 	gnomeicu_tree_user_offline (kontakt);
 
-	tray_update(FALSE);
+	tray_update();
+
+	kontakt->last_seen = time(NULL);
+	kontakt->online_since = 0;
+	kontakt->idle_since = 0;
+	Save_RC();
 
         Shortcut_Update( kontakt );
 }
 
-void User_Online(GSList *contact)
+void User_Online(GSList *contact, gboolean startup)
 {
 #ifdef TRACE_FUNCTION
 	g_print( "User_Online (%s/%s)\n", kontakt->uin, kontakt->nick );
@@ -107,7 +113,7 @@ void User_Online(GSList *contact)
 	g_assert(contact != NULL);
 
 	if( /* enable_online_events && */ kontakt->online_notify &&
-                !kontakt->ignore_list ) 
+                !kontakt->ignore_list) 
 	{
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (GTK_WINDOW (MainData->window),
@@ -120,11 +126,12 @@ void User_Online(GSList *contact)
 		gtk_widget_show (dialog);
 	}
 
-	gnomeicu_event (EV_USERON, contact);
+	if (!startup)
+	  gnomeicu_event (EV_USERON, contact);
 
 	gnomeicu_tree_user_online (kontakt);
 
-	tray_update(FALSE);
+	tray_update();
 
 }
 
@@ -462,6 +469,8 @@ void recv_awaymsg( UIN_T uin, DWORD status, const gchar *message)
                                         GTK_STOCK_OK, GTK_RESPONSE_OK,
                                         NULL);
 
+  gtk_window_set_default_size(GTK_WINDOW(window), 350, 250);
+
   gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_OK);
 
   gtk_window_set_role (GTK_WINDOW (window), "GnomeICU_AwayMessage");
@@ -548,7 +557,7 @@ void start_transfer( UIN_T uin, gchar *msgid, DWORD fileip, WORD fileport)
   }
 }
 
-void set_contact_status (GSList *contact, int status)
+void set_contact_status (GSList *contact, int status, gboolean startup)
 {
 	DWORD last_status;
 
@@ -561,10 +570,64 @@ void set_contact_status (GSList *contact, int status)
 
 	if (last_status == STATUS_OFFLINE &&
 	    kontakt->status != STATUS_OFFLINE)
-	  User_Online (contact);
+	  User_Online (contact, startup);
+
+	kontakt->last_seen = time(NULL);
+	Save_RC();
 
 	gnomeicu_tree_change_status(kontakt);
 
 	Shortcut_Update (kontakt);
 	
 }
+
+void auth_reply(UIN_T uin, int flag, char *reason, time_t msg_time)
+{
+  GtkWidget *dialog;
+  GSList *contact;
+  char *tempmsg;
+  struct tm *msg_tm;
+  STORED_MESSAGE_PTR new_stored_message;
+
+  contact = Find_User (uin);
+
+  if (!contact)
+    return;
+  
+  if (flag) {
+    gnomeicu_tree_user_remove (kontakt);
+    kontakt->wait_auth = FALSE;
+    gnomeicu_tree_user_add (kontakt);
+  } 
+  
+  
+
+  gnomeicu_event( EV_AUTH,contact );
+
+  if (flag)
+    tempmsg = g_strdup_printf(_("%s (%s) has granted you authorization to add him to your contact list%s%s"),
+			      kontakt->nick, kontakt->uin, 
+			      reason ? ": " : "",
+			      reason ? reason : "" );
+  else
+    tempmsg = g_strdup_printf(_("%s (%s) has denied you authorization to add him to your contact list%s%s"),
+			      kontakt->nick, kontakt->uin,
+			      reason ? ": " : "",
+			      reason ? reason : "" );
+
+
+  history_add_incoming( uin, tempmsg, &msg_time );
+  g_free(tempmsg);
+
+  msg_tm = localtime( &msg_time );
+
+  new_stored_message = g_new0( stored_message, 1 );
+  new_stored_message->type = MESSAGE_USER_AUTHD;
+  new_stored_message->message = g_strdup(reason);
+  new_stored_message->data = GINT_TO_POINTER(flag);
+  
+
+  kontakt->stored_messages = g_slist_append( kontakt->stored_messages,
+					     new_stored_message );
+
+}
