@@ -28,6 +28,7 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
 
 static int minimum_font_size = 12;
 
@@ -104,6 +105,7 @@ struct codepage_to_locale {
 };
 
 struct codepage_to_locale ansicpgs[] = {
+    { 943, "SJIS" },
     { 950, "BIG5" },
     { 709, "ASMO_449" },
     { 0, NULL }
@@ -282,8 +284,8 @@ get_charset_for_codepage(int codepage)
 	  g_iconv_close(converter);
 	  return g_strdup(ansicpgs[i].locale);
 	}
-	i++;
       }
+      i++;
     }
     return g_strdup(preferences_get_string(PREFS_GNOMEICU_CHARSET_FALLBACK));
 }
@@ -490,13 +492,12 @@ rtf_parse_charset(RTFParserContext* parser_context, gint charset)
     case 1:
         rtf_set_font_charset(parser_context, parser_context->font_index, parser_context->default_codepage);
         return;
+    case 128: /* ShiftJIS */
+        parser_context->codepage = 943;
+        break;
     case 130: /* Johab */
         parser_context->codepage = 1361;
         break;
-    case 136: /* Big5 */
-    case 1028:
-      parser_context->codepage = 950;
-      break;
     case 134: /* GB2312 */
       parser_context->codepage = 936;
       break;
@@ -594,19 +595,29 @@ rtf_parse_lang(RTFParserContext* parser_context, gint lang)
     {
     case 9:
       /* A user said the person sending with lang=9 was using American WinXP */
-    case 1033:
-    case 1036:
-        /* 1036 was reported as french */
+    case 1033: /* English (US) */
+    case 1036: /* French (France) */
+    case 3084: /* French (Canada) */
+    case 4105: /* English (Canada) */
+    case 1031: /* German ? */
         parser_context->codepage = 1252;
         break;
-    case 1049:
+    case 1049: /* Russian */
         parser_context->codepage = 1251;
         break;
-    case 1028:
+    case 136:  /* Big5 */
+    case 1028: /* Chinese (Taiwan) */
+    case 3076: /* Chinese (Hong Kong) */
         parser_context->codepage = 950;
         break;
+    case 1041: /* Japanese */
+        parser_context->codepage = 943;
+        break;
+    case 1037: /* Hebrew */
+        parser_context->codepage = 1255;
+        break;
         /* ...
-         * Other charsets should be recognized here
+         * Other languages should be recognized here
          * ...
          */
     default:
@@ -663,7 +674,7 @@ rtf_commit_pending_text(RTFParserContext* parser_context)
 					       NULL, NULL, &converterror);
       g_free(charset);
       if (converterror)
-	g_warning("Conversing error: %s\n", converterror->message);
+	g_warning("Conversion error: %s\n", converterror->message);
       g_clear_error(&converterror);
     }
 
@@ -1028,7 +1039,7 @@ rtf_parse_control_word(RTFParserContext* parser_context)
 			"'\\highlight' control word without parameter.");
  	rtf_commit_pending_text( parser_context );
 	attributes->background = param; /* I think this is the background */
-   } else if (! strncmp("fs", word, 2) ) {
+    } else if (! strncmp("fs", word, 2) ) {
 	gchar* tag_name;
 	parser_context->current_pos += 2;
 	if (! rtf_get_int_parameter(parser_context, &param))
@@ -1086,6 +1097,12 @@ rtf_parse_control_word(RTFParserContext* parser_context)
     } else if (! strncmp("\\", word, 1) ) {
 	parser_context->current_pos += 1;
 	g_string_append_c(attributes->text, '\\');
+    } else if (! strncmp("nowidctlpar", word, 11)) {
+        /* ignore widow/orphan control */
+        parser_context->current_pos += 11;
+    } else if (! strncmp("widctlpar", word, 9)) {
+        /* ignore widow/orphan control */
+        parser_context->current_pos += 9;
     } else {
 	rtf_warning(parser_context,
 		    "Unknown control word.");
