@@ -162,7 +162,7 @@ void send_contact_list( GtkWidget *widget, gpointer data )
   g_print( "send_contact_list\n" );
 #endif
 
-  if (!is_connected(GTK_WINDOW(g_object_get_data (G_OBJECT (widget), "dlg")), _("You can not send contacts while being disconnected.")))
+  if (!is_connected(GTK_WINDOW(g_object_get_data (G_OBJECT (widget), "dlg")), _("You can not send contacts while disconnected.")))
     return;
         
   uin = g_object_get_data (G_OBJECT (widget ), "uin" );
