@@ -853,7 +853,8 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
   int sock;
   int version;
   GSList *contact;
-  guint bytes_read;
+  gsize bytes_read = 0;
+  GIOStatus iostatus;
 	
 #ifdef TRACE_FUNCTION
   g_print( "TCPInitChannel(" );
@@ -869,16 +870,19 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
 
   sock = g_io_channel_unix_get_fd( source );
 
-  if( ( bytes_read = read( sock, (char*)&packet_size, 1 ) ) <= 0 ) {
-    close( sock );
-    return FALSE;
-  }
-  
-  if( bytes_read == 1 ) {
-    bytes_read = 0;
-    while( !bytes_read )
-      g_io_channel_read_chars( source, (gchar*)&packet_size + 1,
-                         1, &bytes_read , NULL);
+  while( bytes_read < 2) {
+    gsize read;
+    iostatus = g_io_channel_read_chars( source, (gchar*)&packet_size + bytes_read,
+					2 - bytes_read, &read , NULL);
+    switch (iostatus) {
+    case G_IO_STATUS_NORMAL:
+      bytes_read += read;
+      break;
+    case G_IO_STATUS_ERROR:
+    case G_IO_STATUS_EOF:
+      g_io_channel_shutdown(source, 1, NULL);
+      return FALSE;
+    }
   }
   
 
@@ -886,7 +890,24 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
         
   packet = (BYTE *)g_malloc0( packet_size );
   read( sock, packet, packet_size );
+
+  bytes_read = 0;
+  while( bytes_read < packet_size) {
+    gsize read;
+    iostatus = g_io_channel_read_chars( source, packet + bytes_read,
+					packet_size - bytes_read, &read , NULL);
+    switch (iostatus) {
+    case G_IO_STATUS_NORMAL:
+      bytes_read += read;
+      break;
+    case G_IO_STATUS_ERROR:
+    case G_IO_STATUS_EOF:
+      g_io_channel_shutdown(source, 1, NULL);
+      return FALSE;
+    }
+  }
   
+
   packet_print( packet, packet_size,
                 PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "Init Packet" );
   
@@ -909,7 +930,7 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
     kontakt->used_version = version;
 
   fcntl( sock, F_SETFL, O_NONBLOCK );
-  kontakt->gioc = g_io_channel_unix_new( sock );
+  kontakt->gioc = source;
   kontakt->giocw =
     g_io_add_watch( kontakt->gioc,
                     G_IO_IN | G_IO_ERR | G_IO_HUP,
@@ -1478,7 +1499,7 @@ int TCPFinishConnection( GIOChannel *source, GIOCondition cond, gpointer data )
 int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data )
 {
   BYTE *packet;
-  guint bytes_read;
+  gsize bytes_read;
   int sock;
   GSList *contact;
 
