@@ -22,6 +22,7 @@
 
 /*
  * Copyright (c) 2003 Daniel Romberg
+ * Copyright (c) 2006 Olivier Crte
  */
 
 #include "common.h"
@@ -30,52 +31,38 @@
 #include "util.h"
 #include "icons.h"
 #include "tray.h"
-#include "eggtrayicon.h"
 #include "msg.h"
 #include "showlist.h"
 
 #include <gtk/gtk.h>
 
-EggTrayIcon *tray_icon;
-GtkWidget *tray_icon_image;
-GtkTooltips *tray_icon_tooltip;
+#if GTK_CHECK_VERSION(2,10,0)
 
-static gboolean tray_clicked (GtkWidget *event_box, GdkEventButton *event,
-                              gpointer d);
-static gboolean tray_menu (GtkWidget *event_box, GdkEventButton *event,
-                           gpointer d);
-static void tray_destroyed(GtkObject *object, gpointer d);
-static gboolean app_have_tray (void);
+GtkStatusIcon *status_icon = NULL;
 
-gboolean
-tray_init (void)
+
+static void status_activated  (GtkStatusIcon *status_icon, gpointer user_data);
+static void status_popup (GtkStatusIcon *status_icon, guint button, 
+			  guint activate_time, gpointer user_data);
+
+void tray_init (void)
 {
-	GtkWidget *eventbox;
+	
+  status_icon = gtk_status_icon_new_from_pixbuf(get_pixbuf_for_status (Current_Status));
 
-	tray_icon = egg_tray_icon_new ("GnomeICU");
-	tray_icon_image = gtk_image_new_from_pixbuf (get_pixbuf_for_status (Current_Status));
-	gtk_widget_show (tray_icon_image);
+ 
+  g_signal_connect (G_OBJECT (status_icon), "activate",
+		    G_CALLBACK (status_activated), NULL );
 
-	eventbox = gtk_event_box_new ();
-	gtk_widget_show (eventbox);
-	gtk_container_add (GTK_CONTAINER (eventbox), tray_icon_image);
-	gtk_container_add (GTK_CONTAINER (tray_icon), eventbox);
+  g_signal_connect (G_OBJECT (status_icon), "popup-menu",
+		    G_CALLBACK (status_popup), NULL );
 
-	g_signal_connect (G_OBJECT (eventbox), "button_press_event",
-	                  G_CALLBACK (tray_clicked), NULL );
-
-	g_signal_connect (G_OBJECT(tray_icon), "destroy",
-			  G_CALLBACK (tray_destroyed), NULL);
-
-	gtk_widget_show (GTK_WIDGET (tray_icon));
-
-	tray_icon_tooltip = gtk_tooltips_new ();
-
-	return FALSE;
 }
 
-void
-tray_update (gboolean flash)
+
+
+
+void tray_update (void)
 {
   GSList *ginfo;
   GSList *contact;
@@ -85,88 +72,66 @@ tray_update (gboolean flash)
   gchar *tooltips_str;
   gint online_user = 0;
 
-	if (!tray_icon)
-	  return;
+  if (!status_icon)
+    return;
 
-	contact = Contacts;
+  contact = Contacts;
 
-	while( contact != NULL )
-	{
-		guchar len = g_slist_length( kontakt->stored_messages );
-		if( len ) {
-			if( nextmsg == NULL ) {
-				nextmsg = g_strdup_printf( _("Next message from %s"), kontakt->nick );
-				type = ((STORED_MESSAGE_PTR)kontakt->stored_messages->data)->type;
-			}
-			num_messages += len;
-		}
-		contact = contact->next;
-	}
+  while( contact != NULL ) {
+    guchar len = g_slist_length( kontakt->stored_messages );
+    if( len ) {
+      if( nextmsg == NULL ) {
+	nextmsg = g_strdup_printf( _("Next message from %s"), kontakt->nick );
+	type = ((STORED_MESSAGE_PTR)kontakt->stored_messages->data)->type;
+      }
+      num_messages += len;
+    }
+    contact = contact->next;
+  }
 
-	if( num_messages ) {
-		if( flash )
-			gtk_image_set_from_pixbuf (GTK_IMAGE(tray_icon_image),
-			                get_pixbuf_for_message( type ));
-		else
-			gtk_image_set_from_pixbuf (GTK_IMAGE(tray_icon_image),
-		       	                icon_blank_pixbuf );
-	} else {
-		gtk_image_set_from_pixbuf (GTK_IMAGE (tray_icon_image),
-					   get_pixbuf_for_status (Current_Status));
-	}
-	gtk_widget_show (tray_icon_image);
+  gtk_status_icon_set_blinking(status_icon, num_messages);
+  if (num_messages) {
+    gtk_status_icon_set_from_pixbuf(status_icon,
+				    get_pixbuf_for_message(type));
+  } else {
+    gtk_status_icon_set_from_pixbuf(status_icon,
+				    get_pixbuf_for_status(Current_Status));
+  }
 
-        for (ginfo = Groups; ginfo != NULL; ginfo = ginfo->next)
-          online_user += ((GroupInfo *)ginfo->data)->online_user;
+  for (ginfo = Groups; ginfo != NULL; ginfo = ginfo->next)
+    online_user += ((GroupInfo *)ginfo->data)->online_user;
 
-	if (nextmsg != NULL)
-	  tooltips_str = g_strdup_printf (_("%s\n%d %s\n%d %s\n%s"),
-					  get_status_str (Current_Status),
-					  online_user,
-					  online_user > 1 ? 
-					  _("contacts online") :
-					  _("contact online"),
-					  num_messages, (num_messages > 1) ?
-					  _("new messages"):_("new message"),
-					  nextmsg );
-	else
-	  tooltips_str = g_strdup_printf(_("%s\n%d %s"),
-					 get_status_str (Current_Status),
-					 online_user,
-					 online_user > 1 ? 
-					 _("contacts online"):
-					 _("contact online"));
+  if (nextmsg != NULL)
+    tooltips_str = g_strdup_printf ("%s\n%d %s\n%d %s\n%s",
+				    get_status_str (Current_Status),
+				    online_user,
+				    n_("contact online", 
+				       "contacts online", 
+				       online_user),
+				    num_messages, 
+				    n_("new message",
+				       "new messages",
+				       num_messages),
+				    nextmsg );
+  else
+    tooltips_str = g_strdup_printf("%s\n%d %s",
+				   get_status_str (Current_Status),
+				   online_user,
+				   n_("contact online", 
+				      "contacts online",
+				      online_user));
 
-
-	gtk_tooltips_set_tip (tray_icon_tooltip,
-	                      GTK_WIDGET (tray_icon),
-	                      tooltips_str,
-	                      NULL);
-	g_free(tooltips_str);
+  gtk_status_icon_set_tooltip(status_icon, tooltips_str);
+  g_free(tooltips_str);
 }
 
-gboolean
-tray_clicked (GtkWidget *event_box, GdkEventButton *event, gpointer d)
+
+static void status_activated  (GtkStatusIcon *status_icon, gpointer user_data)
 {
+
   GSList *contact;
   gboolean pending_msg = FALSE;
-  gboolean double_clicked = FALSE;
 
-  /* Right now there is no HIG on how we should handle 'clicking' event
-     for the notification.
-     
-     Current behavior:
-      - double click: show/hide GnomeICU window, or open pending event window
-      - right click: show the status menu
-  */
-
-  if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) {
-    double_clicked == TRUE;
-  } else if (event->button == 3) {
-    /* right click */
-    return tray_menu (event_box, event, d);
-  } else
-    return FALSE; /* nothing match, we return */
 
   /* only reach here when double click occured */
 
@@ -182,24 +147,23 @@ tray_clicked (GtkWidget *event_box, GdkEventButton *event, gpointer d)
   if (pending_msg) {
     show_contact_message (kontakt);
   } else {
-    if (GTK_WIDGET_VISIBLE (app)) {
+    if (GTK_WIDGET_VISIBLE (MainData->window)) {
       /* window visible, hide it */
-      gtk_window_get_position (GTK_WINDOW (app), &MainData->x, &MainData->y);
-      gtk_widget_hide (app);
+      gtk_window_get_position (GTK_WINDOW (MainData->window),
+			       &MainData->x, &MainData->y);
+      gtk_widget_hide (MainData->window);
       MainData->hidden = TRUE;
     } else {
       /* unhide */
-      gtk_window_move (GTK_WINDOW (app), MainData->x, MainData->y);
-      gtk_window_present (GTK_WINDOW(app));
+      gtk_window_move (GTK_WINDOW (MainData->window), MainData->x, MainData->y);
+      gtk_window_present (GTK_WINDOW(MainData->window));
       MainData->hidden = FALSE;
     }
   }
-
-  return TRUE;
 }
 
-gboolean
-tray_menu (GtkWidget *widget, GdkEventButton *event, gpointer d)
+static void status_popup (GtkStatusIcon *status_icon, guint button, 
+			  guint activate_time, gpointer user_data)
 {
 	GtkWidget *status_menu;
 	GtkWidget *item, *image;
@@ -279,50 +243,16 @@ tray_menu (GtkWidget *widget, GdkEventButton *event, gpointer d)
 
 	gtk_widget_show_all (status_menu);
 
-	gtk_menu_popup (GTK_MENU( status_menu), NULL, NULL,
-	                NULL, NULL,
-	                event->button, event->time);
+	gtk_menu_popup (GTK_MENU(status_menu), NULL, NULL,
+	                gtk_status_icon_position_menu, status_icon,
+	                button, activate_time);
 
-	return TRUE;
 }
 
-void
-tray_destroyed(GtkObject *object, gpointer d)
+gboolean
+tray_exists (void)
 {
-
-  tray_icon = NULL;
-
-  tray_init();
-  
-
-
-  /* Show the main window if we dont have the tray */
-  if (!app_have_tray ()) {
-    if (MainData->hidden) {
-      gtk_window_move (GTK_WINDOW (app), MainData->x, MainData->y);
-      MainData->hidden = FALSE;
-    }
-    gtk_window_present (GTK_WINDOW(app));
-  }
-
-
+  return gtk_status_icon_is_embedded(status_icon);
 }
 
-
-static gboolean app_have_tray (void)
-{
-  Screen *xscreen = DefaultScreenOfDisplay (gdk_display);
-  Atom    selection_atom;
-  char   *selection_atom_name;
-	
-  selection_atom_name = g_strdup_printf ("_NET_SYSTEM_TRAY_S%d",
-					 XScreenNumberOfScreen (xscreen));
-  selection_atom = XInternAtom (DisplayOfScreen (xscreen), selection_atom_name, False);
-  g_free (selection_atom_name);
-	
-  if (XGetSelectionOwner (DisplayOfScreen (xscreen), selection_atom)) {
-    return TRUE;
-  } else {
-    return FALSE;
-  }
-}
+#endif /* GTK_CHECK_VERSION(2,10,0) */
