@@ -641,7 +641,8 @@ GList *get_codes( FIELD_T field )
 	for( i = 0; array[i].code != last; i++ )
 		l = g_list_prepend( l, _(array[i].name) );
 	
-	return g_list_reverse( l );
+	//	return g_list_reverse( l );
+	return g_list_sort(l, (GCompareFunc) strcoll);
 }
 
 gchar *get_name( FIELD_T field, WORD code )
@@ -728,7 +729,8 @@ WORD get_code( FIELD_T field, const gchar *name )
 	return array[i].code;
 }
 
-const gchar *get_status_str( DWORD status )
+const gchar *
+get_status_str (DWORD status)
 {
 	if( status == STATUS_OFFLINE )
 		return _("Offline");
