@@ -27,6 +27,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h> 
 #include <netdb.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -39,13 +40,14 @@ gboolean v7_error_handler (GIOChannel *iochannel, GIOCondition condition,
                      V7Connection *conn);
 
 /*  Handlers for the different type of packets */
+void   v7_handle_errors             (Snac *snac);
 void   v7_snac_channel_handler      (V7Packet *packet, V7Connection *conn);
 void   v7_newconn_channel_handler   (V7Packet *packet, V7Connection *conn);
 
 void   v7_make_requests             (V7Connection *conn);
 void   v7_ack_ack_rate              (Snac *snac, V7Connection *conn);
 void   v7_receive_msg               (Snac *snac, V7Connection *conn);
-void   v7_text_message              (UIN_T uin, gchar *here);
+void   v7_text_message              (UIN_T uin, gchar *here, int len);
 void   v7_special_message           (UIN_T uin, gchar *here);
 void   v7_advanced_message          (UIN_T uin, V7Connection *conn,
 				     gchar *here, int len, gchar *begofmes);
@@ -229,6 +231,9 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
   switch (snac->family) {
   case FAMILY_01_GENERIC:
     switch (snac->type) {
+    case F01_BOTH_ERROR:
+      v7_handle_errors(snac);
+      break;
     case F01_SERVER_READY:
       v7_iam_icq(conn);
       break;
@@ -254,6 +259,9 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     break;
   case FAMILY_02_LOCATION:
     switch (snac->type) {
+    case F02_BOTH_ERROR:
+      v7_handle_errors(snac);
+      break;
     case F02_SERVER_RIGHTS_LOCATION: 
       /* We dont understand the data, ignore it */
       break;
@@ -263,6 +271,9 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     break;
   case FAMILY_03_CONTACTLIST:
     switch (snac->type) {
+    case F03_BOTH_ERROR:
+      v7_handle_errors(snac);
+      break;
     case F03_SERVER_CL_INFO:
       /* We dont understand the data, ignore it */
       break;
@@ -278,6 +289,9 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     break;
   case FAMILY_04_MESSAGING:
     switch (snac->type) {
+    case F04_BOTH_ERROR:
+      v7_handle_errors(snac);
+      break;
     case F04_SERVER_RIGHTS_MSG:
       /* We dont understand the data, ignore it */
       break;
@@ -293,6 +307,9 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     break;
   case FAMILY_09_LISTS:
     switch (snac->type) {
+    case F09_BOTH_ERROR:
+      v7_handle_errors(snac);
+      break;
     case F09_SERVER_LIST_RIGHTS:
       /* We dont understand the data, ignore it */
       break;
@@ -308,6 +325,9 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     break;
   case FAMILY_13_SAVED_LIST:
     switch (snac->type) {
+    case F13_BOTH_ERROR:
+      v7_handle_errors(snac);
+      break;
     case F13_SERVER_REQ_RIGHTS_ANS: /* 0x03 */
       /* reply to F13_SERVER_UNKNOWN_1 */
       break;
@@ -343,11 +363,8 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
   case FAMILY_15_SAVED_DATA:
     switch (snac->type) {
     case F15_BOTH_ERROR:
-      switch(CharsBE_2_Word(snac->data)) {
-      case 0x02: /* Rate too fast */
-	g_print("Server says we are sending too fast..\n");
-	break;
-      }
+      v7_handle_errors(snac);
+      break;
     case F15_SERVER_SAVED_DATA:
       v7_saved_data(conn, snac);
     default:
@@ -364,13 +381,27 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     break;
     
   }
-
   g_free (snac->data);
   g_free(snac);
 
 }
 
 
+void v7_handle_errors(Snac *snac)
+{
+  GtkWidget *dialog;
+  
+   dialog = gtk_message_dialog_new(GTK_WINDOW (MainData->window), 
+				   GTK_DIALOG_DESTROY_WITH_PARENT,
+				   GTK_MESSAGE_WARNING,
+				   GTK_BUTTONS_OK,
+				   v7_get_error_message(CharsBE_2_Word(snac->data)));
+
+  gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_widget_destroy(dialog);
+
+}
+
 
 void v7_newconn_channel_handler (V7Packet *packet, V7Connection *conn) 
 {
@@ -426,7 +457,7 @@ void v7_ack_ack_rate    (Snac *snac, V7Connection *conn)
   here = snac->data + 2;
   
   for(i=0; i < classcount; i++) {
-    if ( ((int)here)-((int)snac->data)+33 > snac->datalen) {
+    if ( ((long int)here)-((long int)snac->data)+33 > snac->datalen) {
       g_warning("Bizzare ack rate packet, parsing interupted half way through");
       break;
     }
@@ -485,7 +516,7 @@ void v7_ack_ack_rate    (Snac *snac, V7Connection *conn)
   }
 
   for(i=0; i < classcount; i++) {
-    if ( ((int)here)-((int)snac->data) > snac->datalen) {
+    if ( ((long int)here)-((long int)snac->data) > snac->datalen) {
       g_warning("Bizzare ack rate packet, parsing interupted half way through");
       break;
     }
@@ -528,7 +559,7 @@ void v7_ack_ack_rate    (Snac *snac, V7Connection *conn)
 
 void v7_receive_msg (Snac *snac, V7Connection *conn)
 {
-  gchar *here = snac->data + 8;
+  gchar *here = snac->data + 8; /* skip 8 for cookie */
   WORD msgformat;
   BYTE uinlen;
   UIN_T uin;
@@ -541,37 +572,34 @@ void v7_receive_msg (Snac *snac, V7Connection *conn)
 #endif
 
   msgformat = CharsBE_2_Word(here);
-
-  here += 2;
-  len -= 2;
+ 
+  v7skip(here, len, 2); /* skip format */
+  
+  uinlen = here[0]; 
+  v7skip(here, len, 1); 
   
-  uinlen = here[0];
-  here += 1;
-  len -=1;
 
   uin = g_malloc0(uinlen+1);
   g_memmove(uin, here, uinlen);
 
-  here += uinlen;
-  len -= uinlen;
+  v7skip(here, len, uinlen); /* skip uin */
 
-  here += 2;
-  len -= 2;
+  v7skip(here, len, 2); /* skip sender warning level */
   
-  tlvcount = CharsBE_2_Word(here);
-  here += 2;
-  len -=2;
+  tlvcount = CharsBE_2_Word(here);  
+  v7skip(here, len, 2); /* skip tlv count which should always be 4 */
 
   for(i=0; i<tlvcount; i++) {
+    if (len <= 4)
+      return;
     blah = new_tlv(here); 
-    here += blah->len + 4; 
-    len -=  (blah->len + 4);
+    v7skip(here, len, blah->len + 4); 
     delete_tlv(blah); 
   }
 
   switch(msgformat) {
   case 0x01:
-    v7_text_message(uin, here);
+    v7_text_message(uin, here, len);
     break;
   case 0x02:
     v7_advanced_message(uin, conn, here, len, snac->data);
@@ -587,7 +615,7 @@ void v7_receive_msg (Snac *snac, V7Connection *conn)
   g_free(uin);
 }
 
-void v7_text_message(UIN_T uin, gchar *here)
+void v7_text_message(UIN_T uin, gchar *here, int len)
 {
   
   WORD msglen;
@@ -598,24 +626,30 @@ void v7_text_message(UIN_T uin, gchar *here)
 #ifdef TRACE_FUNCTION
   g_print( "v7_text_message\n" );
 #endif
-  
-  msgtlv = new_tlv(here); 
 
-  if (msgtlv->type == 4 && msgtlv->len == 0) {
-    TLV *temptlv;
-    temptlv = new_tlv(here + 4 + msgtlv->len);
+  if (len < 4) return;
+
+  msgtlv = new_tlv_check(here, len); 
+  if (!msgtlv) return;
+
+  v7skip(here, len, 4 + msgtlv->len);
+
+  /* 
+   * type = 4 == automated response
+   * type = 0x13 == ??? (size=1 and content=6)
+   */
+  while (msgtlv->type != 2) {
     delete_tlv(msgtlv);
-    msgtlv = temptlv;
-  }
+    msgtlv = new_tlv_check(here, len);
+    if (!msgtlv) return;
   
-  if (msgtlv->type != 2) {
-    g_warning(_("TLV is %d, expecting 2\n"), msgtlv->type);
-    return;
+    v7skip(here, len, 4 + msgtlv->len);
+
   }
-    
+
 
   /* I dont know why 3, but it does work */
-  msglen = CharsBE_2_Word(msgtlv->value+ 6 + msgtlv->value[3]); 
+  msglen = CharsBE_2_Word(msgtlv->value+ 6 + CharsBE_2_Word(msgtlv->value+2)); 
   msglen -= 4;                /* to skip the 4 zeros */
   
   if (msglen + 13 > msgtlv->len) { 
@@ -626,7 +660,19 @@ void v7_text_message(UIN_T uin, gchar *here)
   
   msg = g_malloc0(msglen +1);
   
-  g_memmove(msg, msgtlv->value + 12 + msgtlv->value[3], msglen);
+  g_memmove(msg, msgtlv->value + 12 + CharsBE_2_Word(msgtlv->value+2), msglen);
+
+  if (CharsBE_2_Word(msgtlv->value+CharsBE_2_Word(msgtlv->value+2)+8)==2) {
+    gchar *newmsg = NULL;
+
+    newmsg = g_convert(msg, msglen, "UTF-8" ,"UTF-16BE", NULL, NULL, NULL);
+
+    if (newmsg != NULL) {
+      g_free(msg);
+      msg = newmsg;
+    }      
+  }
+      
   
   delete_tlv(msgtlv); 
   
@@ -742,28 +788,47 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
   time_t msg_time;
   gboolean isfileokack=FALSE;
   gboolean isfile = FALSE;
+  int tmplen = len;
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_advanced_message\n" );
 #endif
 
 
-  maintlv = new_tlv(data);
+  here = data;
 
-  if (maintlv->type != 5) {
-    g_warning("Advanced msg has a tlv of a bizzare type: %d\n", maintlv->type);
-    print_data(maintlv->value, maintlv->len);
+  maintlv = new_tlv_check(here, tmplen);
+  if (!maintlv) return;
+
+  v7skip(here, tmplen, 4 + maintlv->len);
+
+  /* There is also type 19 with len=1 and content=11 */
+
+  while (maintlv->type != 5) {
     delete_tlv(maintlv);
-    return;
-  } 
+    maintlv = new_tlv_check(here, len);
+    if (!maintlv) return;
+  
+    v7skip(here, tmplen, 4 + maintlv->len);
+    
+  }
 
   type1 = CharsBE_2_Word (maintlv->value);
 
-  if (type1 == 1) {
-    g_warning("We have an abort request, we dont handle that right now!\n");
+  if (type1 != 0) {
+    switch (type1) {
+    case 1:
+      g_warning("We have an abort request, we dont handle that right now!\n");
+      break;
+    case 2:
+      g_warning("We have a file ack, we dont handle that right now!\n");
+    default:
+      g_warning("We have something else: %d\n", type1);
+    }
+    
     delete_tlv(maintlv);
     return;
-  } 
+  }
   
   here = maintlv->value+26;
   
@@ -789,6 +854,8 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
       break;
     case 0x000F: /* always empty ? */
       break;
+    case 0x0017: /* ??????????? size=2 content=0xFBC1 */
+      break;
     default:
       g_warning("We have a tlv that we dont know of type: %x\n",
                 insidetlv->type);
@@ -819,17 +886,31 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
   case 0x01: /* text message */
     msg_received(uin, msg, msg_time);
     break;
-  case 0x02: /* Chat */
-  case 0x03: /* File .. are you sure? */ 
+  case 0x00: /* Automatic message */
+  case 0xE8:  /* Request for away message */
+  case 0xE9:  /* Request for occupied message */
+  case 0xEA:  /* Request for N/A message */
+  case 0xEB:  /* Request for DND message */
+  case 0xEC:  /* Request for free for chat message */
     break;
-  case 0x1A: /* File */
+  case 0x02: /* Chat */
+  case 0x07: /* Auth reject message */
+  case 0x08: /* Auth accept message */
+  case 0x0d: /* through www.icq.com's web pager */
+  case 0x0e: /* from [uin]@pager.icq.com] */
+  case 0x41: /* some kind of file xfer */
+    break;
+  case 0x03: /* File */ 
+  case 0x1A: /* Extended */
   case 0x13: /* Contacts */
     
     if (isfile) {
       if (isfileokack) {
 	tlv = new_tlv(here+insidetlv->len+4);
-	if (tlv->type == 4 && tlv->len == 4)
-	  fileip = IP_2_DW(tlv->value);
+	if (tlv->type == 4 && tlv->len == 4) {
+	  fileip = IP_2_DW(tlv->value); 
+	  printf( "fileip: %s", inet_ntoa(fileip));
+	}
 	delete_tlv(tlv);
       }
 
@@ -934,12 +1015,6 @@ void v7_advanced_message(UIN_T uin, V7Connection *conn, gchar *data, int len,
 
     authorization_request(uin, msg, msg_time);
     break;
-  case 0xE8:  /* Request for away message */
-  case 0xE9:
-  case 0xEA:
-  case 0xEB:
-  case 0xEC:
-    break;
   default:
     g_warning("Advanced message type not handled, type: %x, text: %s\n",
               msgtype, msg);
@@ -1070,18 +1145,46 @@ void v7_incoming_user (Snac *snac)
 	tempstatus = STATUS_DND;
 	break;
       }
-      set_contact_status (contact, tempstatus);
+      
+      /* the normal invisible status is online+invisible, other
+	 status will be shown as normal */
+      if (tempstatus & STATUS_INVISIBLE &&
+	  tempstatus != STATUS_INVISIBLE)
+	tempstatus &= ~STATUS_INVISIBLE;
+	
+      /* ignore custom status flag */
+      if (tempstatus & STATUS_CUSTOM)
+	tempstatus &= ~STATUS_CUSTOM;
+	
+      set_contact_status (contact, tempstatus, 
+			  snac->packet->conn->readysince + 30 >
+			  time(NULL));
       break;
     case 0x01:
 	if ((CharsBE_2_Word(tlv->value+1) & 0xBFFF) == STATUS_AIM_ONLINE)
-		set_contact_status (contact, STATUS_ONLINE);
+		set_contact_status (contact, STATUS_ONLINE,
+				    snac->packet->conn->readysince + 30 >
+				    time(NULL));
 	else if ((CharsBE_2_Word(tlv->value+1) & 0xBFFF) == STATUS_AIM_AWAY)
-		set_contact_status (contact, STATUS_AWAY);
+		set_contact_status (contact, STATUS_AWAY, 
+				    snac->packet->conn->readysince + 30 >
+				    time(NULL));
 	break;
     case 0x0D: /* Capabilites */
       for (j=0 ; j<=tlv->len ; j+=16)
 	if (!strncmp(tlv->value+j, "\x97\xB1\x27\x51" "\x24\x3C\x43\x34" "\xAD\x22\xD6\xAB" "\xF7\x3F\x14\x92", 16))
 	  kontakt->has_rtf = TRUE;
+      break;
+    case 0x03:
+      kontakt->online_since = CharsBE_2_DW(tlv->value);
+      update_personal_info(kontakt->uin);
+      break;
+    case 0x05:
+      kontakt->member_since = CharsBE_2_DW(tlv->value);
+      update_personal_info(kontakt->uin);
+      break;
+    case 0x04:
+      kontakt->idle_since = time(NULL) - (CharsBE_2_Word(tlv->value)*60);
     }
 
     delete_tlv(tlv); 
@@ -1089,7 +1192,8 @@ void v7_incoming_user (Snac *snac)
 
   if (kontakt->status == STATUS_OFFLINE) {
     status_has_changed = TRUE;
-    set_contact_status (contact, STATUS_ONLINE);
+    set_contact_status (contact, STATUS_ONLINE, 
+			snac->packet->conn->readysince + 30 > time(NULL));
   }
 
   kontakt->last_time = time( NULL );
@@ -2142,7 +2246,7 @@ void  v7_rate_changed (Snac *snac, V7Connection *conn)
   //    gnome_warning_dialog(_("You are sending too fast, please slow down your sending speed."));
   
   if (code == 3)
-    gnome_warning_dialog(_("You are sending messages too fast, your last message has been dropped, please wait 10 seconds and resend. If you continue sending messages at that speed, you will be disconnect by the server. Please give it a break."));
+    gnome_warning_dialog(_("You are sending messages too fast, your last message has been dropped, please wait 10 seconds and resend. If you continue sending messages at that speed, you will be disconnected by the server. Please give it a break."));
   
 }
 
