@@ -192,15 +192,15 @@ void v7_setstatus2(V7Connection *conn, DWORD newstatus)
     IP to zero
   */
     
-  /*   DW_2_IP(directconnectioninfo, our_ip); */
-  /*   DW_2_CharsBE(directconnectioninfo+4, our_port); */
+  DW_2_IP(directconnectioninfo, our_ip);
+  DW_2_CharsBE(directconnectioninfo+4, our_port);
 
   directconnectioninfo[8] = 4; /* is 1 is firewall -- 4 is normal*/
 
 
-  directconnectioninfo[10] = 8; /* protocol v8=2001b v7=2000b */
+  //  directconnectioninfo[10] = 8; /* protocol v8=2001b v7=2000b */
   /* DW_2_CharsBE(directconnectioninfo+15, 0x50); no web front */
-  DW_2_CharsBE(directconnectioninfo+19, 3);
+  //  DW_2_CharsBE(directconnectioninfo+19, 3);
   
 
   add_tlv(tlvs, 0x0C, directconnectioninfo, 37);
@@ -256,7 +256,8 @@ void v7_quit()
     }
     g_list_free(mainconnection->rateclasses);
     
-    g_hash_table_destroy(mainconnection->familyclass);
+    if (mainconnection->familyclass)
+      g_hash_table_destroy(mainconnection->familyclass);
     
     g_free(mainconnection);
     
@@ -1330,10 +1331,15 @@ void v7_client_ready (V7Connection *conn)
     "\x00\x0A\x00\x01\x01\x10\x04\x7B"
     "\x00\x0B\x00\x01\x01\x10\x04\x7B";
 
-  snac_send(conn, ready_data2001, 80, FAMILY_01_GENERIC, F01_CLIENT_READY, NULL,
-	    ANY_ID);
+#ifdef TRACE_FUNCTION
+  g_print ("v7_client_ready\n");
+#endif
+
+  snac_send(conn, ready_data2001, sizeof(ready_data2001),
+            FAMILY_01_GENERIC, F01_CLIENT_READY, NULL, ANY_ID);
 
   conn->status = READY;
+  conn->readysince = time(NULL);
   enable_online_events = TRUE;
   gnomeicu_spinner_stop();
 
@@ -1477,7 +1483,18 @@ void v7_acceptfile(XferInfo *xfer)
     xfer->auto_save_path = preferences_get_string (PREFS_GNOMEICU_DOWNLOAD_PATH);
   }
 
+  printf("OUR IP IS %lu %s\n", xfer->conn->ourip, inet_ntoa(xfer->conn->ourip));
+
   v7_send_file_msg2(xfer->conn, xfer->remote_contact->uin, xfer->msg, 
 		    xfer->port, xfer->conn->ourip, FALSE, xfer->filename,
 		    xfer->total_bytes, xfer->msgid, xfer->downcounter);
 }
+
+void v7_send_idletime(V7Connection *conn, DWORD idletime)
+{
+
+  snac_send(conn, (char*)&idletime, sizeof(DWORD), FAMILY_01_GENERIC, 
+	    F01_CLIENT_SET_IDLE, NULL, ANY_ID);
+
+
+}
