@@ -87,8 +87,8 @@ static gboolean contact_list_inconsistent = FALSE;
 gboolean v7_got_contact_list (V7Connection *conn);
 
 static void snac13_construct_nickname (TLVstack *t, UIN_T uin, guint16 gid,
-				       guint16 uid, const gchar *nick,
-				       gboolean needauth);
+                                       guint16 uid, const gchar *nick,
+                                       gboolean needauth);
 static void snac13_construct_id_ingroup (TLVstack *t, WORD gid);
 static void snac13_construct_visible (TLVstack *t, UIN_T uin, guint16 uid);
 static void snac13_construct_invisible (TLVstack *t, UIN_T uin, guint16 uid);
@@ -115,7 +115,7 @@ v7_request_contacts_list (V7Connection *conn)
 
   /* request rights (0x13, 0x02) */
   snac_send (conn, NULL, 0, FAMILY_13_SAVED_LIST,
-	     F13_CLIENT_REQ_RIGHTS, NULL, ANY_ID);
+             F13_CLIENT_REQ_RIGHTS, NULL, ANY_ID);
 
   if (contacts_resync) {
     list_time_stamp = 0;
@@ -125,14 +125,14 @@ v7_request_contacts_list (V7Connection *conn)
   /* check pref setting, if server list pref not set, check from server*/
   if (!srvlist_exist) {
     snac_send (conn, NULL, 0, FAMILY_13_SAVED_LIST, 
-	       F13_CLIENT_QUERY_SAVED_LIST, NULL, ANY_ID);
+               F13_CLIENT_QUERY_SAVED_LIST, NULL, ANY_ID);
   } else {
     /* send request */
     DW_2_CharsBE (stamp, list_time_stamp);
     Word_2_CharsBE (&stamp[4], record_cnt);
 
     snac_send (conn, stamp, 6, FAMILY_13_SAVED_LIST, 
-	       F13_CLIENT_REQ_SAVED_LIST, NULL, ANY_ID);
+               F13_CLIENT_REQ_SAVED_LIST, NULL, ANY_ID);
   }
 } /* end v7_request_contacts_list() */
 
@@ -150,7 +150,7 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
   static gboolean done_reading_list = FALSE;
   static guint timer_handle = 0;
   gchar *here = snac->data;
-  gchar *rec_str = NULL; /* used to save the BWS string on each record */
+  gchar *xml_str = NULL; /* used to save the BWS string on each record */
   gint rec_count = 0; /* number of contact list record in current snac */
   gint i;
   gint len_str, gid, uid;
@@ -192,11 +192,16 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
   need_reset = FALSE;
 
   for (i = 0; i < rec_count; i++) {
-    /* read the BWS string on each CL record */
+    /* read the BWS string (Group name, UIN for icq, or AIM name) on each CL
+       record */
     len_str = CharsBE_2_Word (here);
     here += 2;
     if (len_str != 0) {
-      rec_str = g_strndup (here, len_str);
+      xml_str = g_markup_escape_text (here, len_str);
+      if (!xml_str) {
+        g_warning ("%s %s\n", "v7snac13.c: unable to allocate space for nickname", "(xml_str)");
+        break;
+      }
       here += len_str;
     }
 
@@ -206,7 +211,6 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
     uid = CharsBE_2_Word (here);
     here += 2;
 
-
     /* read the TLV, find out the type */
     tlv = new_tlv (here);
 
@@ -214,7 +218,7 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
       TLV *tlv2;
       gint tlv_read; /* bytes read inside current tlv */
 
-    case 0:  /* contains nickname, for each user */
+    case 0:  /* TLV 0: nickname, for each user */
       if (gid == 0) {
         /* the contact does not belong to any group, causing problems to
          * many other clients.
@@ -227,28 +231,30 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
       /* g_print ("+++uin_seen: %d\n", uin_seen); */
 
       /* create a user (if not exists) without specifying nickname */
-      if ( (contact = Find_User (rec_str)) == NULL ) {
-        contact = Add_User (rec_str, NULL, TRUE);
+      if ( (contact = Find_User (xml_str)) == NULL ) {
+        contact = Add_User (xml_str, NULL, TRUE);
       }
 
       tlv_read = 0;
       while (tlv_read < tlv->len) {
         tlv2 = new_tlv (tlv->value + tlv_read);
-
         switch (tlv2->type) {
         case 0x131: /* nickname */
           g_free(kontakt->nick);
-          kontakt->nick = g_strdup(tlv2->value);
+          kontakt->nick = g_strdup (tlv2->value);
           break;
         case 0x66: /* with auth */
           kontakt->wait_auth = TRUE;
           break;
         case 0x13a: /* unknown tlv type, icq added after 2003a (jan 2004)*/
-	case 0x137: /* Users email, added by icqlite */
-	case 0x145: /* a message timestamp from icqlite */
-	  break;
+        case 0x137: /* Users email, added by icqlite */
+	case 0x144: /* unknown.. length is 1 .. content is 0x31 */
+        case 0x145: /* a message timestamp from icqlite */
+	case 0x6d: /* Unknown tlv.. comes from Mark (Mac ICQ? Gaim? Other) */
+          break;
         default:
           g_warning ("Unknown tlv type (0x%x) inside type 0", tlv2->type);
+	  packet_print_v7_snac(snac);
           break;
         }
 
@@ -265,7 +271,7 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
     case 1:  /* contains id's (group or uin) */
       /* group id is not zero, contains uin id */
       if (gid) {
-        groups_add (rec_str, gid);
+        groups_add (xml_str, gid);
         group_seen++;
         /* g_print ("***group_seen: %d\n", group_seen); */
       }
@@ -274,33 +280,33 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
       if (tlv->len == 0) {
         uin_seen = 0;
         uin_count = 0;
-	break;
+        break;
       }
 
       tlv2 = new_tlv (tlv->value);
       if (tlv2->type != 0xC8) {
-	g_warning ("expecting TLV(C8): got TLV(0x%x).",tlv2->type);
+        g_warning ("expecting TLV(C8): got TLV(0x%x).",tlv2->type);
       }
       if (gid) {
         uin_seen = 0; /* reset it because we are reading another group now */
         uin_count = tlv2->len / 2;
-        /* g_print ("+++++++++++++++uin_count: %d\n", uin_count); */
+        /* g_print ("uin_count: %d\n", uin_count); */
       } else {
         group_count = tlv2->len / 2;
-        /* g_print ("***************group_count: %d\n",group_count); */
+        /* g_print ("group_count: %d\n",group_count); */
       }
       delete_tlv (tlv2);
       break;
     case 2:  /* zero length; vis to uin */
-      if ( (contact = Find_User (rec_str)) == NULL ) {
-        contact = Add_User (rec_str, NULL, TRUE);
+      if ( (contact = Find_User (xml_str)) == NULL ) {
+        contact = Add_User (xml_str, NULL, TRUE);
       }
       kontakt->vislist_uid = uid;
       kontakt->vis_list = TRUE;
       break;
     case 3:  /* zero length; inv to uin */
-      if ( (contact = Find_User (rec_str)) == NULL ) {
-        contact = Add_User (rec_str, NULL, TRUE);
+      if ( (contact = Find_User (xml_str)) == NULL ) {
+        contact = Add_User (xml_str, NULL, TRUE);
       }
       kontakt->invlist_uid = uid;
       kontakt->invis_list = TRUE;
@@ -322,28 +328,43 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
     case 9:  /* unknown meaning, ignore */
       break;
     case 0xE:  /* zero length; ignore to uin */
-      if ( (contact = Find_User (rec_str)) == NULL ) {
-        contact = Add_User (rec_str, NULL, TRUE);
+      if ( (contact = Find_User (xml_str)) == NULL ) {
+        contact = Add_User (xml_str, NULL, TRUE);
       }
       kontakt->ignorelist_uid = uid;
       kontakt->ignore_list = TRUE;
       break;
+    case 0xF:  /* store LastUpdateDate, ignore */
+      break;
     case 0x11: /* found when user used lite.icq.com before server list exists */
       break;
     case 0x13:  /* CL first import time, not really useful to us, ignore */
       break;
+    case 0x14:  /* Own  icon information  its an avatar ID stored as text */
+      break;
+    case 0x15:  /* Recent buddy info */
+      break;
+    case 0x19:  /* Unknown ? */
+      /* Contains a 0x006d TLV which a 64bit number ? */
+      /* neither gaim nor miranda know about it as of
+       * feb 19, 2006 */
+      break;
+    case 0x1d: /* Unknown emnpty */
+      break;
     default:
-      g_warning ("Unknown TLV from server contact list: type 0x%x.\n\n"
-                 "Please send the following packet dump to the mailing-list "
-                 "for debugging, thanks.\n\n",tlv->type);
+      g_warning ("Unknown TLV from server contact list: type 0x%x. It also "
+                 "contains this string \"%s\".\n\n"
+                 "Please send the following packet dump to \n"
+		 "http://bugzilla.gnome.org/enter_bug.cgi?product=GnomeICU \n"
+                 "for debugging, thanks.\n\n", tlv->type, xml_str);
       packet_print_v7_snac (snac);
       break;
     }
 
     here += TLV_HEADER + tlv->len;
 
-    g_free (rec_str);
-    rec_str = NULL;
+    g_free (xml_str);
+    xml_str = NULL;
     delete_tlv (tlv);
   } /* end for(rec_count) */
 
@@ -384,7 +405,7 @@ v7_read_contacts_list (Snac *snac, V7Connection *conn)
     }
 
     snac_send (conn, NULL, 0, FAMILY_13_SAVED_LIST, 
-	       F13_CLIENT_RDY_TO_USE_LIST, NULL, ANY_ID);
+               F13_CLIENT_RDY_TO_USE_LIST, NULL, ANY_ID);
     v7_set_user_info (conn);
     v7_add_icbm (conn);
     v7_setstatus2 (conn, Current_Status);
@@ -432,10 +453,10 @@ v7_begin_CL_edit (V7Connection *conn, gboolean migrate)
 
   if (migrate)
     snac_send (conn, data, 4, FAMILY_13_SAVED_LIST,
-	       F13_CLIENT_START_MOD_LIST, NULL, ANY_ID);
+               F13_CLIENT_START_MOD_LIST, NULL, ANY_ID);
   else
     snac_send (conn, NULL, 0, FAMILY_13_SAVED_LIST,
-	       F13_CLIENT_START_MOD_LIST, NULL, ANY_ID);
+               F13_CLIENT_START_MOD_LIST, NULL, ANY_ID);
 
 }
 
@@ -525,7 +546,7 @@ v7_migrate_contacts_list (V7Connection *conn)
     id = contact_gen_uid ();
 
     snac13_construct_nickname (tlvs, kontakt->uin, main_gid, id, kontakt->nick,
-			       FALSE);
+                               FALSE);
 
     kontakt->gid = main_gid;
     kontakt->uid = id;
@@ -605,7 +626,7 @@ v7_migrate_contacts_list (V7Connection *conn)
  */
 guint
 v7_addto_contacts_list (V7Connection *conn, UIN_T uin, const gchar *nick,
-			guint gid, gboolean needauth)
+                        guint gid, gboolean needauth)
 {
   guint uid;
   TLVstack *tlvs;
@@ -646,7 +667,7 @@ v7_send_grant_auth (V7Connection *conn, UIN_T uin)
   snac13_construct_authreq (tlvs, uin, NULL);
   
   snac_send (conn, tlvs->beg, tlvs->len, FAMILY_13_SAVED_LIST,
-	     F13_CLIENT_AUTH_REQUEST_1, NULL, ANY_ID);
+             F13_CLIENT_AUTH_REQUEST_1, NULL, ANY_ID);
 
   free_tlvstack (tlvs);
 }
@@ -685,13 +706,13 @@ v7_snac13_check_srv_ack (V7Connection *conn, Snac *snac)
         contactdata = g_new0 (Contact_Member, 1);
         contactdata->info = g_new0 (USER_INFO_STRUCT, 1);
 
-	/* Only set for newly created contacts */
-	contactdata->status = STATUS_OFFLINE;
+        /* Only set for newly created contacts */
+        contactdata->status = STATUS_OFFLINE;
       } else {
         contactdata = contact->data;
       }
       if (contact)
-	gnomeicu_tree_user_remove (contactdata);
+        gnomeicu_tree_user_remove (contactdata);
 
       g_free(contactdata->uin);
       contactdata->uin = g_strdup (snac13_state.uin);
@@ -718,6 +739,8 @@ v7_snac13_check_srv_ack (V7Connection *conn, Snac *snac)
       /* refresh the gui */
 
       gnomeicu_tree_user_add (contactdata);
+      if (contactdata->status != STATUS_OFFLINE)
+        gnomeicu_tree_user_online (contactdata);
 
       nick = contactdata->nick;
       break;
@@ -867,7 +890,7 @@ v7_ask_uin_for_auth (V7Connection *conn, gchar *authmsg, UIN_T uin)
   tlvs = new_tlvstack (NULL, 0);
   snac13_construct_authreq (tlvs, uin, authmsg);
   snac_send (conn, tlvs->beg, tlvs->len, FAMILY_13_SAVED_LIST,
-	     F13_CLIENT_AUTH_REQUEST_2, NULL, ANY_ID);
+             F13_CLIENT_AUTH_REQUEST_2, NULL, ANY_ID);
   free_tlvstack (tlvs);
 
   /* send the auth grant again, if the user has intended */
@@ -966,7 +989,7 @@ v7_grant_auth_request (V7Connection *conn, UIN_T uin)
   add_nontlv (t, "\x1\0\0\0\0", 5);
 
   snac_send (conn, t->beg, t->len, FAMILY_13_SAVED_LIST,
-	     F13_CLIENT_SEND_GRANT_AUTH, NULL, ANY_ID);
+             F13_CLIENT_SEND_GRANT_AUTH, NULL, ANY_ID);
 
   free_tlvstack (t);
 }
@@ -978,8 +1001,10 @@ void
 v7_user_grant_auth (Snac *snac)
 {
   gchar *here = snac->data;
-  GSList *contact;
   UIN_T uin;
+  int flag;
+  char *reason = NULL, *r2=NULL;
+  int strlen;
 
   /* with this flag, we have to skip 8 bytes first */
   if (snac->flags[0] && 0x80)
@@ -989,16 +1014,23 @@ v7_user_grant_auth (Snac *snac)
   uin = g_strndup (here+1, here[0]);
   here = here + here[0] + 1;
 
-  /* FIXME this part should be in a seperate function */
-  contact = Find_User (uin);
-  
-  if (contact) {
-    gnomeicu_tree_user_remove (kontakt);
-    kontakt->wait_auth = FALSE;
-    gnomeicu_tree_user_add (kontakt);
+  flag = here[0];
+  here++;
+
+  strlen = CharsBE_2_Word(here);
+  here += 2;
+  if (strlen) {
+    r2 = g_strndup(here, strlen);
+    if (r2) {
+      reason = convert_to_utf8(r2);
+      g_free(r2);
+    }
   }
 
+  auth_reply(uin, flag, reason, time(NULL));
+
   g_free (uin);
+  g_free (reason);
 }
 
 /*
@@ -1332,21 +1364,21 @@ void
 snac13_send_add_to_list (V7Connection *conn, const TLVstack *t)
 {
   snac_send (conn, t->beg, t->len, FAMILY_13_SAVED_LIST,
-	     F13_CLIENT_ADD_TO_LIST, NULL, ANY_ID);
+             F13_CLIENT_ADD_TO_LIST, NULL, ANY_ID);
 }
 
 void
 snac13_send_update_list (V7Connection *conn, const TLVstack *t)
 {
   snac_send (conn, t->beg, t->len, FAMILY_13_SAVED_LIST,
-	     F13_CLIENT_UPDATE_LIST, NULL, ANY_ID);
+             F13_CLIENT_UPDATE_LIST, NULL, ANY_ID);
 }
 
 void
 snac13_send_remove_from_list (V7Connection *conn, const TLVstack *t)
 {
   snac_send (conn, t->beg, t->len, FAMILY_13_SAVED_LIST,
-	     F13_CLIENT_REMOVE_FROM_LIST, NULL, ANY_ID);
+             F13_CLIENT_REMOVE_FROM_LIST, NULL, ANY_ID);
 }
 
 void
