@@ -26,7 +26,7 @@
  *
  * <?xml?>
  * <gnomeicu xmlns="http://gnomeicu.sourceforge.net/xmlcontact">
- *   <version>0.94</version>
+ *   <version>0.94.1</version>
  *   <icq>
  *     <uin>11111111</uin>
  *     <password>mypasswd</password>
@@ -49,6 +49,7 @@
  *          <notify/> <!-- online notification, gnomeicu client side feature -->
  *          <uin>23434343</uin>
  *          <nick>foo</nick>
+ *          <lastseen>321313</lastseen> <!-- time_t when last seen (at UTC) -->
  *       </user>
  *       <user> <!-- anonymous users, usually on your ignore list -->
  *          <status uid="3454">ignore</status>
@@ -75,7 +76,7 @@
 #include "v7snac13.h"
 #include "util.h"
 
-#define CONTACT_FILE_VERSION "0.94"
+#define CONTACT_FILE_VERSION "0.94.1"
 
 static void xmlcontact_save_icq (xmlNodePtr icuNode);
 static void icq_contact_set_status (xmlNodePtr node, const gchar *status,
@@ -101,6 +102,7 @@ typedef enum {
   PARSER_ICQ_USER,
   PARSER_ICQ_WAIT_AUTH,
   PARSER_ICQ_NOTIFY,
+  PARSER_ICQ_LAST_SEEN,
   PARSER_UNKNOWN
 } ParserState;
 
@@ -144,6 +146,7 @@ struct _XMLContactParseState {
   gint c_vis_id, c_inv_id, c_ign_id;
   gboolean wait_auth;
   gboolean notify;
+  time_t last_seen;
 };
 
 /* initialize the state */
@@ -267,6 +270,7 @@ xmlContactStartEle (XMLContactParseState *state, const xmlChar *name,
   case PARSER_ICQ_NOTIFY:
   case PARSER_ICQ_PASSWORD:
   case PARSER_ICQ_WEB_PRESENCE:
+  case PARSER_ICQ_LAST_SEEN:
     /* should have no element after these states */
     state->prev_state = state->state;
     state->state = PARSER_UNKNOWN;
@@ -275,6 +279,7 @@ xmlContactStartEle (XMLContactParseState *state, const xmlChar *name,
   case PARSER_ICQ_CONTACT: /* inside <contact> */
     if (!strcmp (name, "user")) {
       state->state = PARSER_ICQ_USER;
+
       state->gid = 0;
       state->uid = 0;
       /* read the user gid and uid */
@@ -297,6 +302,8 @@ xmlContactStartEle (XMLContactParseState *state, const xmlChar *name,
       state->state = PARSER_ICQ_UIN;
     } else if (!strcmp (name, "nick")) {
       state->state = PARSER_ICQ_NICK;
+    } else if (!strcmp (name, "lastseen")) {
+      state->state = PARSER_ICQ_LAST_SEEN;
     } else if (!strcmp (name, "status")) {
       state->state = PARSER_ICQ_STATUS;
       /* parse status uid */
@@ -330,6 +337,7 @@ xmlContactEndEle (XMLContactParseState *state, const xmlChar *name)
 {
   CONTACT_PTR contactdata;
   GSList *contact;
+  gint len;
 
   switch (state->state) {
   case PARSER_UNKNOWN:
@@ -379,6 +387,7 @@ xmlContactEndEle (XMLContactParseState *state, const xmlChar *name)
     g_string_truncate (state->content, 0);
     break;
   case PARSER_ICQ_NICK:
+    len = strlen (state->content->str);
     if (state->owner) {
       state->state = PARSER_ICQ;
       /*g_message ("owner nick is: %s", state->content->str);*/
@@ -389,6 +398,12 @@ xmlContactEndEle (XMLContactParseState *state, const xmlChar *name)
     }
     g_string_truncate (state->content, 0);
     break;
+  case PARSER_ICQ_LAST_SEEN:
+    if (state->content->len)
+      state->last_seen = strtoul(state->content->str, NULL, 10);
+    state->state = PARSER_ICQ_USER;
+    g_string_truncate (state->content, 0);
+    break;
   case PARSER_ICQ_REQUIRE_AUTH:
     state->state = PARSER_ICQ;
     if (!strcmp (state->content->str, "true")) {
@@ -490,6 +505,7 @@ xmlContactEndEle (XMLContactParseState *state, const xmlChar *name)
     contactdata->confirmed = TRUE;
     contactdata->wait_auth = state->wait_auth;
     contactdata->online_notify = state->notify;
+    contactdata->last_seen = state->last_seen;
 
     contact = Find_User (contactdata->uin);
     while (contact && kontakt->gid == 0) {
@@ -510,6 +526,7 @@ xmlContactEndEle (XMLContactParseState *state, const xmlChar *name)
     state->c_ign_id = 0;
     state->wait_auth = FALSE;
     state->notify = FALSE;
+    state->last_seen = 0;
 
     break;
   case PARSER_ICQ_WAIT_AUTH:
@@ -534,7 +551,8 @@ xmlContactChars (XMLContactParseState *state, const xmlChar *chars, gint len)
       state->state == PARSER_ICQ_STATUS ||
       state->state == PARSER_ICQ_GROUP ||
       state->state == PARSER_ICQ_TIMESTAMP ||
-      state->state == PARSER_ICQ_TOTALRECORD)
+      state->state == PARSER_ICQ_TOTALRECORD ||
+      state->state == PARSER_ICQ_LAST_SEEN)
     for (i = 0; i < len; i++)
       g_string_append_c (state->content, chars[i]);
 }
@@ -677,12 +695,12 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
 
   xmlNewChild (icqNode, NULL, "uin", our_info->uin);
 	
-  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, passwd);
+  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, g_markup_escape_text(passwd, strlen(passwd)));
   xmlNewChild (icqNode, NULL, "password", xmlEncoded);
   free (xmlEncoded);
   xmlEncoded = NULL;
 	
-  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, our_info->nick);
+  xmlEncoded = xmlEncodeEntitiesReentrant (NULL, g_markup_escape_text(our_info->nick, strlen(our_info->nick)));
   xmlNewChild (icqNode, NULL, "nick", xmlEncoded);
   free (xmlEncoded);
   xmlEncoded = NULL;
@@ -710,8 +728,7 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
   /* save groups */
   tlist = Groups;
   while (tlist) {
-    child = xmlNewChild (icqNode, NULL, "group",
-                         ((GroupInfo *)tlist->data)->name);
+    child = xmlNewChild (icqNode, NULL, "group", g_markup_escape_text(((GroupInfo *)tlist->data)->name, strlen(((GroupInfo *)tlist->data)->name)));
 
     tmpstr = g_strdup_printf ("%d", ((GroupInfo *)tlist->data)->gid);
     xmlSetProp (child, "gid", tmpstr);
@@ -780,7 +797,15 @@ xmlcontact_save_icq (xmlNodePtr icuNode)
     g_free (tmpstr);
 
     /* nick */
-    xmlNewChild (userNode, NULL, "nick", kontakt->nick);
+    xmlNewChild (userNode, NULL, "nick", g_markup_escape_text(kontakt->nick, strlen(kontakt->nick)));
+
+    /* Last seen online */
+    if (kontakt->status != STATUS_OFFLINE)
+      kontakt->last_seen = time(NULL);
+    tmpstr = g_strdup_printf("%ld", kontakt->last_seen);
+    xmlNewChild (userNode, NULL, "lastseen", tmpstr);
+    g_free (tmpstr);
+
   }
 }
 
